#version 130

#include "settings.glsl"

varying vec2 TexCoords;
uniform float viewWidth, viewHeight;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex3;

const bool colortex3Clear = false;

#if AUTO_EXPOSURE == 1
const bool colortex0MipmapEnabled = true;
#endif

float pw = 1.0 / viewWidth;
float ph = 1.0 / viewHeight;

void AutoExposure(inout vec3 color, inout float exposure, float tempExposure) {
	float exposureLod = log2(viewHeight * AUTO_EXPOSURE_RADIUS);
	
	exposure = length(texture2DLod(colortex0, vec2(0.5), exposureLod).rgb);
	exposure = max(exposure, 0.0001);
	
	color /= tempExposure + 0.125;
}

void Tonemap(inout vec3 color) {
	color *= exp2(2.0 + EXPOSURE);

	color = color / vec3(TONEMAP_WHITE), vec3(1.0 / TONEMAP_WHITE);

	color = clamp(color, vec3(0.0), vec3(1.0));
}

void main() {
	vec3 color = texture2D(colortex0, TexCoords).rgb;
	
	#if AUTO_EXPOSURE == 1
	float tempExposure = texture2D(colortex3, vec2(pw, ph)).r;
    #endif
	
	vec3 temporalColor = vec3(0.0);

    #if AUTO_EXPOSURE == 1
	float exposure = 1.0;
	AutoExposure(color, exposure, tempExposure);
    #endif
	
	Tonemap(color);
	
	float temporalData = 0.0;
	
	#if AUTO_EXPOSURE == 1
	if (TexCoords.x < 2.0 * pw && TexCoords.y < 2.0 * ph)
		temporalData = mix(tempExposure, sqrt(exposure), AUTO_EXPOSURE_SPEED);
	#endif
	
	/* DRAWBUFFERS:43 */
	gl_FragData[0] = vec4(color, 1.0);
	gl_FragData[1] = vec4(temporalData, temporalColor);
}
